/*
  (C) 7.7.2002  Petr Lastovicka
*/
//---------------------------------------------------------------------------
#include <windows.h>
#pragma hdrstop
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#ifndef M_PI
  #define M_PI        3.14159265358979323846
#endif
//---------------------------------------------------------------------------
const int edY=9, edH=20, x1Y=46, x2Y=68, x3Y=90;
HDC dc;
HWND hWin, ed[4];
long double coef[4];
int width;
//---------------------------------------------------------------------------
void print(int y, char *format, ...)
{
 char buf[128];
 va_list va;
 va_start(va,format);
 int n = vsprintf(buf,format,va);
 SetBkMode(dc, TRANSPARENT);
 TextOut(dc, 10, y, buf, n);
 va_end(va);
}
//---------------------------------------------------------------------------
typedef long double real;

const real e = 1e-16;

inline bool nul(real x)
{
 return fabsl(x)<e;
}

inline real sqr(real x)
{
 return x*x;
}

real cbrt(real x)
{
 const real third = 1 / 3.0L;
 return x>=0 ? powl(x, third) : -powl(-x, third);
}
//---------------------------------------------------------------------------
void cubic()
{
 real t,u,v,s;
 real a=coef[0],b=coef[1],c=coef[2],d=coef[3];

 if(!nul(a)){
  u=-b/3/a;
  v=c/a;
  t=u*u*u-(u*v+d/a)/2;
  s=t*t+(v/3-u*u)*sqr(v/3-u*u);
  if(s<-e){
    s=2*sqrtl(u*u-v/3);
    t=acosl(t*8/s/s/s)/3;
    print(x1Y,"x1= %.16LG", cosl(t)*s+u);
    print(x2Y,"x2 = %.16LG", cosl(t+2*M_PI/3)*s+u);
    print(x3Y,"x3 = %.16LG", cosl(t+4*M_PI/3)*s+u);
  }else if(s>e){
    v=cbrt(t+sqrtl(s));
    t=cbrt(t-sqrtl(s));
    print(x1Y,"x1 = %.16LG", v+t+u);
    print(x2Y,"x2,x3 =  %.16LG +- i *  %.16LG",
      -(v+t)/2+u, (v-t)*sqrtl(3)/2);
  }else if(nul(sqr(b)-3*a*c)){
    print(x1Y,"x = %.16LG", u);
  }else{
    print(x1Y,"x1= %.16LG", u-cbrt(t));
    print(x2Y,"x2 = %.16LG", u+cbrt(t)*2);
  }
 }else if(!nul(b)){
  t=-c/2/b;
  u= t*t-d/b;
  if(u>e){
    print(x1Y,"x1= %.16LG", t+sqrtl(u));
    print(x2Y,"x2 = %.16LG", t-sqrtl(u));
  }else if(u<-e){
    print(x1Y,"x1,x2 = %.16LG +- i * %.16LG", t, sqrtl(-u));
  }else{
    print(x1Y,"x= %.16LG", t);
  }
 }else if(!nul(c)){
    print(x1Y,"x= %.16LG", -d/c);
 }
}
//---------------------------------------------------------------------------
LRESULT CALLBACK MainWndProc(HWND hWnd, UINT msg, WPARAM wP, LPARAM lP)
{
 static PAINTSTRUCT ps;
 char buf[32];
 int i;
 RECT rc;
 int w;

 switch (msg) {
 case WM_COMMAND:
  switch(LOWORD(wP)){
   case 0:
    for(i=0; i<4; i++){
     if((HWND)lP==ed[i]){
      SendMessage((HWND)lP,WM_GETTEXT,sizeof(buf),(LPARAM)buf);
      for(char *s=buf; *s; s++){
        char c=*s;
        if(c==',') *s='.';
        else if((c<'0' || c>'9') && c!='e' && c!='E'
           && c!='-' && c!='.'){
          *s=0;
          SendMessage((HWND)lP,WM_SETTEXT,0,(LPARAM)buf);
          SendMessage((HWND)lP,EM_SETSEL,32,32);
        }
      }
      coef[i]= _atold(buf);
     }
    }
    GetClientRect(hWnd, &rc);
    rc.top = x1Y;
    InvalidateRect(hWnd,&rc,TRUE);
   break;
   case 1: //ENTER
   case 2: //ESCAPE
    SetFocus(GetNextDlgTabItem(hWnd,GetFocus(),LOWORD(wP)-1));
    SendMessage(GetFocus(),EM_SETSEL,0,-1);
   break;
  }
 break;

 case WM_PAINT:
  dc=BeginPaint(hWnd,&ps);
  cubic();
  EndPaint(hWnd, &ps);
 break;

 case WM_SIZE:
  width=LOWORD(lP);
  w= (width-20)/4;
  for(i=0; i<4; i++){
    SetWindowPos(ed[i],0, 10+i*w,edY, w-5,edH, SWP_NOZORDER);
  }
 break;

 case WM_GETMINMAXINFO:
  ((POINT far *)lP)[3].x = 250;
  ((POINT far *)lP)[3].y = 150;
 break;

 case WM_DESTROY:
  PostQuitMessage(0);
 break;

 default:
  return DefWindowProc(hWnd, msg, wP, lP);
}
return 0;
}
//---------------------------------------------------------------------------
int pascal WinMain(HINSTANCE hInstance,HINSTANCE hPrevInst,LPSTR,int cmdShow)
{
 WNDCLASS wc;
 MSG msg;
 int i;

 wc.style= 0;
 wc.lpfnWndProc= MainWndProc;
 wc.cbClsExtra= 0;
 wc.cbWndExtra= 0;
 wc.hInstance= hInstance;
 wc.hIcon= LoadIcon(0, IDI_APPLICATION);
 wc.hCursor= LoadCursor(NULL, IDC_ARROW);
 wc.hbrBackground= GetSysColorBrush(COLOR_BTNFACE);
 wc.lpszMenuName= 0;
 wc.lpszClassName= "CubicWCLS";
 if(!hPrevInst && !RegisterClass(&wc)) return 1;

 hWin = CreateWindow(
   "CubicWCLS", " a*X^3 + b*X^2 + c*X + d = 0",
   WS_OVERLAPPEDWINDOW,
   CW_USEDEFAULT,CW_USEDEFAULT,
   430, 150,
   NULL, NULL, hInstance, NULL);
 if(!hWin) return 2;

 for(i=0; i<4; i++){
   ed[i]= CreateWindow(
   "EDIT", "", WS_CHILD|WS_VISIBLE|WS_BORDER|WS_TABSTOP,
   0,0,9,9, hWin, NULL, hInstance, NULL);
 }
 SetFocus(ed[0]);
 ShowWindow(hWin, cmdShow);

 while(GetMessage(&msg, NULL, 0, 0)==TRUE){
   if(!IsDialogMessage(hWin, &msg)){
    DispatchMessage(&msg);
   }
 }
 return msg.wParam;
}
//---------------------------------------------------------------------------

